
// Package lexer is generated by GoGLL. Do not edit.
package lexer

import (
	// "fmt"
	"io/ioutil"
	"strings"
	"unicode"

	"github.com/ChrisTrenkamp/xsel/grammar/token"
)

type state int

const nullState state = -1


// Lexer contains both the input slice of runes and the slice of tokens
// parsed from the input
type Lexer struct {
	// I is the input slice of runes
	I      []rune

	// Tokens is the slice of tokens constructed by the lexer from I
	Tokens []*token.Token
}

/*
NewFile constructs a Lexer created from the input file, fname. 

If the input file is a markdown file NewFile process treats all text outside
code blocks as whitespace. All text inside code blocks are treated as input text.

If the input file is a normal text file NewFile treats all text in the inputfile
as input text.
*/
func NewFile(fname string) *Lexer {
	buf, err := ioutil.ReadFile(fname)
	if err != nil {
		panic(err)
	}
	input := []rune(string(buf))
	if strings.HasSuffix(fname, ".md") {
		loadMd(input)
	}
	return New(input)
}

func loadMd(input []rune) {
	i := 0
	text := true
	for i < len(input) {
		if i <= len(input)-3 && input[i] == '`' && input[i+1] == '`' && input[i+2] == '`' {
			text = !text
			for j := 0; j < 3; j++ {
				input[i+j] = ' '
			}
			i += 3
		}
		if i < len(input) {
			if text {
				if input[i] == '\n' {
					input[i] = '\n'
				} else {
					input[i] = ' '
				}
			}
			i += 1
		}
	}
}

/*
New constructs a Lexer from a slice of runes. 

All contents of the input slice are treated as input text.
*/
func New(input []rune) *Lexer {
	lex := &Lexer{
		I:      input,
		Tokens: make([]*token.Token, 0, 2048),
	}
	lext := 0
	for lext < len(lex.I) {
		for lext < len(lex.I) && unicode.IsSpace(lex.I[lext]) {
			lext++
		}
		if lext < len(lex.I) {
			tok := lex.scan(lext)
			lext = tok.Rext()
			if !tok.Suppress() {
				lex.addToken(tok)
			}
		}
	}
	lex.add(token.EOF, len(input), len(input))
	return lex
}

func (l *Lexer) scan(i int) *token.Token {
	// fmt.Printf("lexer.scan(%d)\n", i)
	s, typ, rext := nullState, token.Error, i+1
	if i < len(l.I) {
		// fmt.Printf("  rext %d, i %d\n", rext, i)
		s = nextState[0](l.I[i])
	}
	for s != nullState {
		if rext >= len(l.I) {
			typ = accept[s]
			s = nullState
		} else {
			typ = accept[s]
			s = nextState[s](l.I[rext])
			if s != nullState || typ == token.Error {
				rext++
			}
		}
	}
	tok := token.New(typ, i, rext, l.I)
	// fmt.Printf("  %s\n", tok)
	return tok
}

func escape(r rune) string {
	switch r {
	case '"':
		return "\""
	case '\\':
		return "\\\\"
	case '\r':
		return "\\r"
	case '\n':
		return "\\n"
	case '\t':
		return "\\t"
	}
	return string(r)
}

// GetLineColumn returns the line and column of rune[i] in the input
func (l *Lexer) GetLineColumn(i int) (line, col int) {
	line, col = 1, 1
	for j := 0; j < i; j++ {
		switch l.I[j] {
		case '\n':
			line++
			col = 1
		case '\t':
			col += 4
		default:
			col++
		}
	}
	return
}

// GetLineColumnOfToken returns the line and column of token[i] in the imput
func (l *Lexer) GetLineColumnOfToken(i int) (line, col int) {
	return l.GetLineColumn(l.Tokens[i].Lext())
}

// GetString returns the input string from the left extent of Token[lext] to
// the right extent of Token[rext]
func (l *Lexer) GetString(lext, rext int) string {
	return string(l.I[l.Tokens[lext].Lext():l.Tokens[rext].Rext()])
}

func (l *Lexer) add(t token.Type, lext, rext int) {
	l.addToken(token.New(t, lext, rext, l.I))
}

func (l *Lexer) addToken(tok *token.Token) {
	l.Tokens = append(l.Tokens, tok)
}

func any(r rune, set []rune) bool {
	for _, r1 := range set {
		if r == r1 {
			return true
		}
	}
	return false
}

func not(r rune, set []rune) bool {
	for _, r1 := range set {
		if r == r1 {
			return false
		}
	}
	return true
}

var accept = []token.Type{ 
	token.Error, 
	token.Error, 
	token.Error, 
	token.Error, 
	token.T_1, 
	token.T_2, 
	token.T_3, 
	token.T_4, 
	token.T_5, 
	token.T_6, 
	token.T_7, 
	token.T_9, 
	token.T_29, 
	token.T_11, 
	token.T_13, 
	token.T_15, 
	token.T_16, 
	token.T_18, 
	token.T_19, 
	token.T_20, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_47, 
	token.Error, 
	token.T_36, 
	token.T_0, 
	token.T_31, 
	token.Error, 
	token.T_44, 
	token.Error, 
	token.T_8, 
	token.T_10, 
	token.T_12, 
	token.T_14, 
	token.T_17, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_38, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_46, 
	token.T_44, 
	token.T_44, 
	token.T_36, 
	token.T_23, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_30, 
	token.T_36, 
	token.T_34, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.Error, 
	token.T_46, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_37, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_43, 
	token.T_45, 
	token.T_46, 
	token.T_36, 
	token.T_36, 
	token.T_25, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_39, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_26, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_21, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_24, 
	token.T_36, 
	token.T_32, 
	token.T_35, 
	token.T_40, 
	token.T_36, 
	token.T_36, 
	token.T_27, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_22, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_33, 
	token.T_41, 
	token.T_36, 
	token.T_28, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_36, 
	token.T_42, 
}

var nextState = []func(r rune) state{ 
	// Set0
	func(r rune) state {
		switch { 
		case r == '!':
			return 1 
		case r == '"':
			return 2 
		case r == '\'':
			return 3 
		case r == '(':
			return 4 
		case r == ')':
			return 5 
		case r == '*':
			return 6 
		case r == '+':
			return 7 
		case r == ',':
			return 8 
		case r == '-':
			return 9 
		case r == '.':
			return 10 
		case r == '/':
			return 11 
		case r == '0':
			return 12 
		case r == '1':
			return 12 
		case r == '2':
			return 12 
		case r == '3':
			return 12 
		case r == '4':
			return 12 
		case r == '5':
			return 12 
		case r == '6':
			return 12 
		case r == '7':
			return 12 
		case r == '8':
			return 12 
		case r == '9':
			return 12 
		case r == ':':
			return 13 
		case r == '<':
			return 14 
		case r == '=':
			return 15 
		case r == '>':
			return 16 
		case r == '@':
			return 17 
		case r == '[':
			return 18 
		case r == ']':
			return 19 
		case r == 'a':
			return 20 
		case r == 'c':
			return 21 
		case r == 'd':
			return 22 
		case r == 'f':
			return 23 
		case r == 'm':
			return 24 
		case r == 'n':
			return 25 
		case r == 'o':
			return 26 
		case r == 'p':
			return 27 
		case r == 's':
			return 28 
		case r == 't':
			return 29 
		case r == '|':
			return 30 
		case any(r, []rune{'$'}):
			return 31 
		case unicode.IsLetter(r):
			return 32 
		}
		return nullState
	}, 
	// Set1
	func(r rune) state {
		switch { 
		case r == '=':
			return 33 
		}
		return nullState
	}, 
	// Set2
	func(r rune) state {
		switch { 
		case r == '"':
			return 34 
		case r == '\\':
			return 35 
		case not(r, []rune{'"','\\'}):
			return 2 
		}
		return nullState
	}, 
	// Set3
	func(r rune) state {
		switch { 
		case r == '\'':
			return 36 
		case r == '\\':
			return 37 
		case not(r, []rune{'\''}):
			return 3 
		}
		return nullState
	}, 
	// Set4
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set5
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set6
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set7
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set8
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set9
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set10
	func(r rune) state {
		switch { 
		case r == '.':
			return 38 
		}
		return nullState
	}, 
	// Set11
	func(r rune) state {
		switch { 
		case r == '/':
			return 39 
		}
		return nullState
	}, 
	// Set12
	func(r rune) state {
		switch { 
		case r == '0':
			return 12 
		case r == '1':
			return 12 
		case r == '2':
			return 12 
		case r == '3':
			return 12 
		case r == '4':
			return 12 
		case r == '5':
			return 12 
		case r == '6':
			return 12 
		case r == '7':
			return 12 
		case r == '8':
			return 12 
		case r == '9':
			return 12 
		}
		return nullState
	}, 
	// Set13
	func(r rune) state {
		switch { 
		case r == ':':
			return 40 
		}
		return nullState
	}, 
	// Set14
	func(r rune) state {
		switch { 
		case r == '=':
			return 41 
		}
		return nullState
	}, 
	// Set15
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set16
	func(r rune) state {
		switch { 
		case r == '=':
			return 42 
		}
		return nullState
	}, 
	// Set17
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set18
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set19
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set20
	func(r rune) state {
		switch { 
		case r == 'n':
			return 43 
		case r == 't':
			return 44 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set21
	func(r rune) state {
		switch { 
		case r == 'h':
			return 46 
		case r == 'o':
			return 47 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set22
	func(r rune) state {
		switch { 
		case r == 'e':
			return 48 
		case r == 'i':
			return 49 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set23
	func(r rune) state {
		switch { 
		case r == 'o':
			return 50 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set24
	func(r rune) state {
		switch { 
		case r == 'o':
			return 51 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set25
	func(r rune) state {
		switch { 
		case r == 'a':
			return 52 
		case r == 'o':
			return 53 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set26
	func(r rune) state {
		switch { 
		case r == 'r':
			return 54 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set27
	func(r rune) state {
		switch { 
		case r == 'a':
			return 55 
		case r == 'r':
			return 56 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set28
	func(r rune) state {
		switch { 
		case r == 'e':
			return 57 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set29
	func(r rune) state {
		switch { 
		case r == 'e':
			return 58 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set30
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set31
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 59 
		}
		return nullState
	}, 
	// Set32
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set33
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set34
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set35
	func(r rune) state {
		switch { 
		case any(r, []rune{'"','\\','n','r','t'}):
			return 2 
		}
		return nullState
	}, 
	// Set36
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set37
	func(r rune) state {
		switch { 
		case r == '\'':
			return 60 
		case r == '\\':
			return 37 
		case not(r, []rune{'\''}):
			return 3 
		case any(r, []rune{'\'','\\','n','r','t'}):
			return 61 
		}
		return nullState
	}, 
	// Set38
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set39
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set40
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set41
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set42
	func(r rune) state {
		switch { 
		}
		return nullState
	}, 
	// Set43
	func(r rune) state {
		switch { 
		case r == 'c':
			return 62 
		case r == 'd':
			return 63 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set44
	func(r rune) state {
		switch { 
		case r == 't':
			return 64 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set45
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set46
	func(r rune) state {
		switch { 
		case r == 'i':
			return 65 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set47
	func(r rune) state {
		switch { 
		case r == 'm':
			return 66 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set48
	func(r rune) state {
		switch { 
		case r == 's':
			return 67 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set49
	func(r rune) state {
		switch { 
		case r == 'v':
			return 68 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set50
	func(r rune) state {
		switch { 
		case r == 'l':
			return 69 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set51
	func(r rune) state {
		switch { 
		case r == 'd':
			return 70 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set52
	func(r rune) state {
		switch { 
		case r == 'm':
			return 71 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set53
	func(r rune) state {
		switch { 
		case r == 'd':
			return 72 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set54
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set55
	func(r rune) state {
		switch { 
		case r == 'r':
			return 73 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set56
	func(r rune) state {
		switch { 
		case r == 'e':
			return 74 
		case r == 'o':
			return 75 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set57
	func(r rune) state {
		switch { 
		case r == 'l':
			return 76 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set58
	func(r rune) state {
		switch { 
		case r == 'x':
			return 77 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set59
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 59 
		case any(r, []rune{':'}):
			return 78 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 79 
		}
		return nullState
	}, 
	// Set60
	func(r rune) state {
		switch { 
		case r == '\'':
			return 36 
		case r == '\\':
			return 37 
		case not(r, []rune{'\''}):
			return 3 
		}
		return nullState
	}, 
	// Set61
	func(r rune) state {
		switch { 
		case r == '\'':
			return 60 
		case r == '\\':
			return 37 
		case not(r, []rune{'\''}):
			return 3 
		case any(r, []rune{'\'','\\','n','r','t'}):
			return 61 
		}
		return nullState
	}, 
	// Set62
	func(r rune) state {
		switch { 
		case r == 'e':
			return 80 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set63
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set64
	func(r rune) state {
		switch { 
		case r == 'r':
			return 81 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set65
	func(r rune) state {
		switch { 
		case r == 'l':
			return 82 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set66
	func(r rune) state {
		switch { 
		case r == 'm':
			return 83 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set67
	func(r rune) state {
		switch { 
		case r == 'c':
			return 84 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set68
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set69
	func(r rune) state {
		switch { 
		case r == 'l':
			return 85 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set70
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set71
	func(r rune) state {
		switch { 
		case r == 'e':
			return 86 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set72
	func(r rune) state {
		switch { 
		case r == 'e':
			return 87 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set73
	func(r rune) state {
		switch { 
		case r == 'e':
			return 88 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set74
	func(r rune) state {
		switch { 
		case r == 'c':
			return 89 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set75
	func(r rune) state {
		switch { 
		case r == 'c':
			return 90 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set76
	func(r rune) state {
		switch { 
		case r == 'f':
			return 91 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set77
	func(r rune) state {
		switch { 
		case r == 't':
			return 92 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set78
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 93 
		}
		return nullState
	}, 
	// Set79
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 79 
		case any(r, []rune{':'}):
			return 78 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 79 
		}
		return nullState
	}, 
	// Set80
	func(r rune) state {
		switch { 
		case r == 's':
			return 94 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set81
	func(r rune) state {
		switch { 
		case r == 'i':
			return 95 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set82
	func(r rune) state {
		switch { 
		case r == 'd':
			return 96 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set83
	func(r rune) state {
		switch { 
		case r == 'e':
			return 97 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set84
	func(r rune) state {
		switch { 
		case r == 'e':
			return 98 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set85
	func(r rune) state {
		switch { 
		case r == 'o':
			return 99 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set86
	func(r rune) state {
		switch { 
		case r == 's':
			return 100 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set87
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set88
	func(r rune) state {
		switch { 
		case r == 'n':
			return 101 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set89
	func(r rune) state {
		switch { 
		case r == 'e':
			return 102 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set90
	func(r rune) state {
		switch { 
		case r == 'e':
			return 103 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set91
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set92
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set93
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 93 
		}
		return nullState
	}, 
	// Set94
	func(r rune) state {
		switch { 
		case r == 't':
			return 104 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set95
	func(r rune) state {
		switch { 
		case r == 'b':
			return 105 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set96
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set97
	func(r rune) state {
		switch { 
		case r == 'n':
			return 106 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set98
	func(r rune) state {
		switch { 
		case r == 'n':
			return 107 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set99
	func(r rune) state {
		switch { 
		case r == 'w':
			return 108 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set100
	func(r rune) state {
		switch { 
		case r == 'p':
			return 109 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set101
	func(r rune) state {
		switch { 
		case r == 't':
			return 110 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set102
	func(r rune) state {
		switch { 
		case r == 'd':
			return 111 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set103
	func(r rune) state {
		switch { 
		case r == 's':
			return 112 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set104
	func(r rune) state {
		switch { 
		case r == 'o':
			return 113 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set105
	func(r rune) state {
		switch { 
		case r == 'u':
			return 114 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set106
	func(r rune) state {
		switch { 
		case r == 't':
			return 115 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set107
	func(r rune) state {
		switch { 
		case r == 'd':
			return 116 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set108
	func(r rune) state {
		switch { 
		case r == 'i':
			return 117 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set109
	func(r rune) state {
		switch { 
		case r == 'a':
			return 118 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set110
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set111
	func(r rune) state {
		switch { 
		case r == 'i':
			return 119 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set112
	func(r rune) state {
		switch { 
		case r == 's':
			return 120 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set113
	func(r rune) state {
		switch { 
		case r == 'r':
			return 121 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set114
	func(r rune) state {
		switch { 
		case r == 't':
			return 122 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set115
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set116
	func(r rune) state {
		switch { 
		case r == 'a':
			return 123 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set117
	func(r rune) state {
		switch { 
		case r == 'n':
			return 124 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set118
	func(r rune) state {
		switch { 
		case r == 'c':
			return 125 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set119
	func(r rune) state {
		switch { 
		case r == 'n':
			return 126 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set120
	func(r rune) state {
		switch { 
		case r == 'i':
			return 127 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set121
	func(r rune) state {
		switch { 
		case r == '-':
			return 128 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 128 
		}
		return nullState
	}, 
	// Set122
	func(r rune) state {
		switch { 
		case r == 'e':
			return 129 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set123
	func(r rune) state {
		switch { 
		case r == 'n':
			return 130 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set124
	func(r rune) state {
		switch { 
		case r == 'g':
			return 131 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set125
	func(r rune) state {
		switch { 
		case r == 'e':
			return 132 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set126
	func(r rune) state {
		switch { 
		case r == 'g':
			return 133 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set127
	func(r rune) state {
		switch { 
		case r == 'n':
			return 134 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set128
	func(r rune) state {
		switch { 
		case r == 'o':
			return 135 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set129
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set130
	func(r rune) state {
		switch { 
		case r == 't':
			return 136 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set131
	func(r rune) state {
		switch { 
		case r == '-':
			return 137 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 137 
		}
		return nullState
	}, 
	// Set132
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set133
	func(r rune) state {
		switch { 
		case r == '-':
			return 138 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 138 
		}
		return nullState
	}, 
	// Set134
	func(r rune) state {
		switch { 
		case r == 'g':
			return 139 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set135
	func(r rune) state {
		switch { 
		case r == 'r':
			return 140 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set136
	func(r rune) state {
		switch { 
		case r == '-':
			return 141 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 141 
		}
		return nullState
	}, 
	// Set137
	func(r rune) state {
		switch { 
		case r == 's':
			return 142 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set138
	func(r rune) state {
		switch { 
		case r == 's':
			return 143 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set139
	func(r rune) state {
		switch { 
		case r == '-':
			return 144 
		case unicode.IsLetter(r):
			return 32 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 144 
		}
		return nullState
	}, 
	// Set140
	func(r rune) state {
		switch { 
		case r == '-':
			return 145 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 145 
		}
		return nullState
	}, 
	// Set141
	func(r rune) state {
		switch { 
		case r == 'o':
			return 146 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set142
	func(r rune) state {
		switch { 
		case r == 'i':
			return 147 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set143
	func(r rune) state {
		switch { 
		case r == 'i':
			return 148 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set144
	func(r rune) state {
		switch { 
		case r == 'i':
			return 149 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set145
	func(r rune) state {
		switch { 
		case r == 's':
			return 150 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set146
	func(r rune) state {
		switch { 
		case r == 'r':
			return 151 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set147
	func(r rune) state {
		switch { 
		case r == 'b':
			return 152 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set148
	func(r rune) state {
		switch { 
		case r == 'b':
			return 153 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set149
	func(r rune) state {
		switch { 
		case r == 'n':
			return 154 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set150
	func(r rune) state {
		switch { 
		case r == 'e':
			return 155 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set151
	func(r rune) state {
		switch { 
		case r == '-':
			return 156 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 156 
		}
		return nullState
	}, 
	// Set152
	func(r rune) state {
		switch { 
		case r == 'l':
			return 157 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set153
	func(r rune) state {
		switch { 
		case r == 'l':
			return 158 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set154
	func(r rune) state {
		switch { 
		case r == 's':
			return 159 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set155
	func(r rune) state {
		switch { 
		case r == 'l':
			return 160 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set156
	func(r rune) state {
		switch { 
		case r == 's':
			return 161 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set157
	func(r rune) state {
		switch { 
		case r == 'i':
			return 162 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set158
	func(r rune) state {
		switch { 
		case r == 'i':
			return 163 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set159
	func(r rune) state {
		switch { 
		case r == 't':
			return 164 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set160
	func(r rune) state {
		switch { 
		case r == 'f':
			return 165 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set161
	func(r rune) state {
		switch { 
		case r == 'e':
			return 166 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set162
	func(r rune) state {
		switch { 
		case r == 'n':
			return 167 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set163
	func(r rune) state {
		switch { 
		case r == 'n':
			return 168 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set164
	func(r rune) state {
		switch { 
		case r == 'r':
			return 169 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set165
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set166
	func(r rune) state {
		switch { 
		case r == 'l':
			return 170 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set167
	func(r rune) state {
		switch { 
		case r == 'g':
			return 171 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set168
	func(r rune) state {
		switch { 
		case r == 'g':
			return 172 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set169
	func(r rune) state {
		switch { 
		case r == 'u':
			return 173 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set170
	func(r rune) state {
		switch { 
		case r == 'f':
			return 174 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set171
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set172
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set173
	func(r rune) state {
		switch { 
		case r == 'c':
			return 175 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set174
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set175
	func(r rune) state {
		switch { 
		case r == 't':
			return 176 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set176
	func(r rune) state {
		switch { 
		case r == 'i':
			return 177 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set177
	func(r rune) state {
		switch { 
		case r == 'o':
			return 178 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set178
	func(r rune) state {
		switch { 
		case r == 'n':
			return 179 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
	// Set179
	func(r rune) state {
		switch { 
		case unicode.IsLetter(r):
			return 45 
		case any(r, []rune{'-','.','0','1','2','3','4','5','6','7','8','9','_','·','̀','́','̂','̃','̄','̅','̆','̇','̈','̉','̊','̋','̌','̍','̎','̏','̐','̑','̒','̓','̔','̕','̖','̗','̘','̙','̚','̛','̜','̝','̞','̟','̠','̡','̢','̣','̤','̥','̦','̧','̨','̩','̪','̫','̬','̭','̮','̯','̰','̱','̲','̳','̴','̵','̶','̷','̸','̹','̺','̻','̼','̽','̾','̿','̀','́','͂','̓','̈́','ͅ','͠','͡','·','҃','҄','҅','҆','֑','֒','֓','֔','֕','֖','֗','֘','֙','֚','֛','֜','֝','֞','֟','֠','֡','֣','֤','֥','֦','֧','֨','֩','֪','֫','֬','֭','֮','֯','ְ','ֱ','ֲ','ֳ','ִ','ֵ','ֶ','ַ','ָ','ֹ','ֻ','ּ','ֽ','ֿ','ׁ','ׂ','ׄ','ً','ٌ','ٍ','َ','ُ','ِ','ّ','ْ','٠','١','٢','٣','٤','٥','٦','٧','٨','٩','ٰ','ۖ','ۗ','ۘ','ۙ','ۚ','ۛ','ۜ','۝','۞','۟','۠','ۡ','ۢ','ۣ','ۤ','ۧ','ۨ','۪','۫','۬','ۭ','۰','۱','۲','۳','۴','۵','۶','۷','۸','۹','ँ','ं','ः','़','ा','ि','ी','ु','ू','ृ','ॄ','ॅ','ॆ','े','ै','ॉ','ॊ','ो','ौ','्','॑','॒','॓','॔','ॢ','ॣ','०','१','२','३','४','५','६','७','८','९','ঁ','ং','ঃ','়','া','ি','ী','ু','ূ','ৃ','ৄ','ে','ৈ','ো','ৌ','্','ৗ','ৢ','ৣ','০','১','২','৩','৪','৫','৬','৭','৮','৯','ਂ','਼','ਾ','ਿ','ੀ','ੁ','ੂ','ੇ','ੈ','ੋ','ੌ','੍','੦','੧','੨','੩','੪','੫','੬','੭','੮','੯','ੰ','ੱ','ઁ','ં','ઃ','઼','ા','િ','ી','ુ','ૂ','ૃ','ૄ','ૅ','ે','ૈ','ૉ','ો','ૌ','્','૦','૧','૨','૩','૪','૫','૬','૭','૮','૯','ଁ','ଂ','ଃ','଼','ା','ି','ୀ','ୁ','ୂ','ୃ','େ','ୈ','ୋ','ୌ','୍','ୖ','ୗ','୦','୧','୨','୩','୪','୫','୬','୭','୮','୯','ஂ','ா','ி','ீ','ு','ூ','ெ','ே','ை','ொ','ோ','ௌ','்','ௗ','௧','௨','௩','௪','௫','௬','௭','௮','௯','ఁ','ం','ః','ా','ి','ీ','ు','ూ','ృ','ౄ','ె','ే','ై','ొ','ో','ౌ','్','ౕ','ౖ','౦','౧','౨','౩','౪','౫','౬','౭','౮','౯','ಂ','ಃ','ಾ','ಿ','ೀ','ು','ೂ','ೃ','ೄ','ೆ','ೇ','ೈ','ೊ','ೋ','ೌ','್','ೕ','ೖ','೦','೧','೨','೩','೪','೫','೬','೭','೮','೯','ം','ഃ','ാ','ി','ീ','ു','ൂ','ൃ','െ','േ','ൈ','ൊ','ോ','ൌ','്','ൗ','൦','൧','൨','൩','൪','൫','൬','൭','൮','൯','ั','ิ','ี','ึ','ื','ุ','ู','ฺ','็','่','้','๊','๋','์','ํ','๎','๐','๑','๒','๓','๔','๕','๖','๗','๘','๙','ັ','ິ','ີ','ຶ','ື','ຸ','ູ','ົ','ຼ','່','້','໊','໋','໌','ໍ','໐','໑','໒','໓','໔','໕','໖','໗','໘','໙','༘','༙','༠','༡','༢','༣','༤','༥','༦','༧','༨','༩','༵','༷','༹','༾','༿','ཱ','ི','ཱི','ུ','ཱུ','ྲྀ','ཷ','ླྀ','ཹ','ེ','ཻ','ོ','ཽ','ཾ','ཿ','ྀ','ཱྀ','ྂ','ྃ','྄','྆','྇','ྐ','ྑ','ྒ','ྒྷ','ྔ','ྕ','ྗ','ྙ','ྚ','ྛ','ྜ','ྜྷ','ྞ','ྟ','ྠ','ྡ','ྡྷ','ྣ','ྤ','ྥ','ྦ','ྦྷ','ྨ','ྩ','ྪ','ྫ','ྫྷ','ྭ','ྱ','ྲ','ླ','ྴ','ྵ','ྶ','ྷ','ྐྵ','⃐','⃑','⃒','⃓','⃔','⃕','⃖','⃗','⃘','⃙','⃚','⃛','⃜','⃡','℮','ↀ','ↁ','ↂ','〇','〡','〢','〣','〤','〥','〦','〧','〨','〩','〪','〫','〬','〭','〮','〯','゙','゚'}):
			return 45 
		}
		return nullState
	}, 
}
